-----------------------------------------------------------------------
-- Upvalued Lua API.
-----------------------------------------------------------------------
local _G = getfenv(0)

local math = _G.math
local string = _G.string
local table = _G.table

local pairs = _G.pairs
local tonumber = _G.tonumber
local tostring = _G.tostring

-----------------------------------------------------------------------
-- AddOn namespace.
-----------------------------------------------------------------------
local ADDON_NAME, private = ...

local LibStub = _G.LibStub
local LibToast = LibStub("LibToast-1.0")

local MessageToaster = LibStub('AceAddon-3.0'):NewAddon(ADDON_NAME, 'AceConsole-3.0')

_G["MessageToaster"] = {}
-----------------------------------------------------------------------
-- Code-y stuff
-----------------------------------------------------------------------

local options = { 
    name = "MessageToaster",
    handler = MessageToaster,
    type = "group",
    args = {
        text = {
            type = "description",
            name = "Use the command |cff00ffff/mttest|r to send a test notification to yourself",
        },

        playSounds = {
            type = "toggle",
            name = "Play Sounds",
            desc = "Whether or not you want a notification sound to be played when you receive a whisper",
            get = "GetPlaySounds",
            set = "SetPlaySounds",
        },
        showToasts = {
            type = "toggle",
            name = "Show Notification Toast",
            desc = "Whether or not you want the notification toast to popup when you receive a whisper",
            get = "GetShowToasts",
            set = "SetShowToasts",
        },
    },
}

local defaults = {
    global = {
        playSounds = true,
        showToasts = true,
    },
}

function MessageToaster:PlayNotificationSound() 
  if (self.db.global.playSounds == true) then
    PlaySoundFile("Interface\\AddOns\\MessageToaster\\Sounds\\NotificationSound.ogg", "master")
  end
end

local function CloseToast()
end

function MessageToaster:SetPlaySounds(info, var)
  self.db.global.playSounds = var
end

function MessageToaster:GetPlaySounds(info)
  return self.db.global.playSounds
end

function MessageToaster:GetShowToasts(info)
  return self.db.global.showToasts
end

function MessageToaster:SetShowToasts(info, var)
  self.db.global.showToasts = var
end

function MessageToaster:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0"):HandleCommand("mtoast", "MessageToaster", input)
    end
end

function MessageToaster:SpawnToast(template, ...)
  if (self.db.global.showToasts) then 
    LibToast:Spawn(template, ...)
  end
end

function MessageToaster:OnInitialize()
  -- never used aceaddon before so forgive the bad form and stuff

  self.db = LibStub("AceDB-3.0"):New("MessageToasterDB", defaults, true)

  LibStub("AceConfig-3.0"):RegisterOptionsTable("MessageToaster", options)
  self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MessageToaster", "MessageToaster")
  self:RegisterChatCommand("mtoast", "ChatCommand")
  self:RegisterChatCommand("messagetoaster", "ChatCommand")
  
--  self.db.global.playSounds = true
--  self.db.global.showToasts = true

end

function MessageToaster:OnEnable()
  -- never used aceaddon before so forgive the bad form and stuff

  LibToast:Register("TestNotification", function(toast, ...)
    toast:SetTitle("Example Toast")
    toast:SetText(...)
    toast:SetIconTexture([[Interface\FriendsFrame\StatusIcon-Online]])
    toast:SetUrgencyLevel("emergency")

  end)

  LibToast:Register("MessageToast", function(toast, message, sender, ...)
    toast:SetTitle(sender)
    toast:SetText(message .. "\n")
    toast:SetIconTexture([[Interface\FriendsFrame\UI-Toast-BroadcastIcon]])

    --toast:MakePersistent()
  end)

  LibToast:Register("IconTest", function(toast, msg)
    toast:SetTitle("Example Toast For Icon")
    toast:SetText(msg)
    toast:SetIconTexture("Interface\\FriendsFrame\\" .. msg)
    toast:MakePersistent()
    --toast:SetPrimaryCallback(_G.OKAY, CloseToast)
  end)

  MessageToaster:RegisterChatCommand("mttest", function(args)
      MessageToaster:PlayNotificationSound()
      MessageToaster:SpawnToast("TestNotification", "Sample Text")--LibToast:Spawn("TestNotification", "Sample Text")
    end)

  MessageToaster:RegisterChatCommand("mtexample", function(args)
      MessageToaster:SpawnToast("MessageToast", tostring("Example message to demonstrate addon"), "PM: YourBNetFriend")
    end)

  local messageTrackingFrame = CreateFrame("Frame", "messageTrackingFrame")
  MessageToaster.messageTrackingFrame = messageTrackingFrame

  messageTrackingFrame:RegisterEvent("CHAT_MSG_BN_WHISPER")
  messageTrackingFrame:RegisterEvent("CHAT_MSG_WHISPER")

  messageTrackingFrame:SetScript("OnEvent", function(self, event, message, sender, ...)
    --print("|cff00ffffMessageToaster|r " .. tostring(sender) .. " : " .. tostring(message))
    MessageToaster:PlayNotificationSound()
    MessageToaster:SpawnToast("MessageToast", tostring(message), "PM: " .. tostring(sender))--LibToast:Spawn("MessageToast", tostring(message), "PM: " .. tostring(sender))
  end)

  print("|cff00ffffMessageToaster|r:|cffFF7400 Has loaded!");
  print("|cff00ffffMessageToaster|r:|cffFF7400 Created by Sawuasfoiythl")
  print("|cff00ffffMessageToaster|r:|cffFF7400 Use /mtoast or Interface->Addons->MessageToaster|r for settings")
end

function MessageToaster:OnDisable()
  -- never used aceaddon before so forgive the bad form and stuff
end